@@ -1,26 +1,27 @@
 """
-@author: Thomas Delaet <thomas@delaet.org>
+:author: Thomas Delaet <thomas@delaet.org>
 """
-import velbus
 import json
 import base64
+import velbus
 
 
 class Message(object):
-    # pylint: disable-msg=R0921,R0904
+    # pylint: disable-msg=R0904
     """
     Base Velbus message
     """
 
-    def __init__(self):
+    def __init__(self, address=None):
         self.priority = None
         self.address = None
         self.rtr = False
         self.wait_after_send = 100
+        self.set_defaults(address)
 
     def set_attributes(self, priority, address, rtr):
         """
-        @return: None
+        :return: None
         """
         assert isinstance(priority, int)
         assert isinstance(address, int)
@@ -33,26 +34,34 @@ class Message(object):
 
     def populate(self, priority, address, rtr, data):
         """
-        @return: None
+        :return: None
         """
         raise NotImplementedError
 
     def set_defaults(self, address):
         """
-        @return: None
+        Set defaults
+
+        If a message has different than low priority or NO_RTR set,
+        then this method needs override in subclass
+
+        :return: None
         """
-        raise NotImplementedError
+        if address is not None:
+            self.set_address(address)
+        self.set_low_priority()
+        self.set_no_rtr()
 
     def set_address(self, address):
         """
-        @return: None
+        :return: None
         """
         assert isinstance(address, int)
         self.address = address
 
     def to_binary(self):
         """
-        @return: bytes
+        :return: bytes
         """
         pre_checksum_data = self.__checksum_data()
         checksum = velbus.checksum(pre_checksum_data)
@@ -60,13 +69,13 @@ class Message(object):
 
     def to_base64(self):
         """
-        @return: str
+        :return: str
         """
         return base64.b64encode(self.to_binary())
 
     def __checksum_data(self):
         """
-        @return: bytes
+        :return: bytes
         """
         data_bytes = self.data_to_binary()
         if self.rtr:
@@ -79,19 +88,27 @@ class Message(object):
 
     def data_to_binary(self):
         """
-        @return: bytes
+        :return: bytes
         """
         raise NotImplementedError
 
     def to_json_basic(self):
+        """
+        Create JSON structure with generic attributes
+
+        :return: dict
+        """
         return {'name': self.__class__.__name__, 'priority': self.priority,
                 'address': self.address, 'rtr': self.rtr}
 
     def to_json(self):
         """
-        @return: str
+        Dump object structure to JSON
+
+        This method should be overridden in subclasses to include more than just generic attributes
+
+        :return: str
         """
-        # FIXME: Implement in subclasses
         return json.dumps(self.to_json_basic())
 
     def __str__(self):
@@ -99,12 +116,12 @@ class Message(object):
 
     def byte_to_channels(self, byte):
         """
-        @return: list(int)
+        :return: list(int)
         """
         # pylint: disable-msg=R0201
         assert isinstance(byte, int)
-        assert(byte >= 0)
-        assert(byte < 256)
+        assert byte >= 0
+        assert byte < 256
         result = []
         for offset in range(0, 8):
             if byte & (1 << offset):
@@ -113,7 +130,7 @@ class Message(object):
 
     def channels_to_byte(self, channels):
         """
-        @return: int
+        :return: int
         """
         # pylint: disable-msg=R0201
         assert isinstance(channels, list)
@@ -125,7 +142,7 @@ class Message(object):
 
     def byte_to_channel(self, byte):
         """
-        @return: int
+        :return: int
         """
         assert isinstance(byte, int)
         channels = self.byte_to_channels(byte)
@@ -134,7 +151,7 @@ class Message(object):
 
     def needs_valid_channel(self, channel, maximum):
         """
-        @return: None
+        :return: None
         """
         assert isinstance(channel, int)
         assert isinstance(maximum, int)
@@ -143,13 +160,13 @@ class Message(object):
 
     def parser_error(self, message):
         """
-        @return: None
+        :return: None
         """
         raise velbus.ParserError(self.__class__.__name__ + " " + message)
 
     def needs_rtr(self, rtr):
         """
-        @return: None
+        :return: None
         """
         assert isinstance(rtr, bool)
         if not rtr:
@@ -157,13 +174,13 @@ class Message(object):
 
     def set_rtr(self):
         """
-        @return: None
+        :return: None
         """
         self.rtr = True
 
     def needs_no_rtr(self, rtr):
         """
-        @return: None
+        :return: None
         """
         assert isinstance(rtr, bool)
         if rtr:
@@ -171,13 +188,13 @@ class Message(object):
 
     def set_no_rtr(self):
         """
-        @return: None
+        :return: None
         """
         self.rtr = False
 
     def needs_low_priority(self, priority):
         """
-        @return: None
+        :return: None
         """
         assert isinstance(priority, int)
         if priority != velbus.LOW_PRIORITY:
@@ -185,13 +202,13 @@ class Message(object):
 
     def set_low_priority(self):
         """
-        @return: None
+        :return: None
         """
         self.priority = velbus.LOW_PRIORITY
 
     def needs_high_priority(self, priority):
         """
-        @return: None
+        :return: None
         """
         assert isinstance(priority, int)
         if priority != velbus.HIGH_PRIORITY:
@@ -199,28 +216,43 @@ class Message(object):
 
     def set_high_priority(self):
         """
-        @return: None
+        :return: None
         """
         self.priority = velbus.HIGH_PRIORITY
 
+    def needs_firmware_priority(self, priority):
+        """
+        :return: None
+        """
+        assert isinstance(priority, int)
+        if priority != velbus.FIRMWARE_PRIORITY:
+            self.parser_error("needs firmware priority set")
+
+    def set_firmware_priority(self):
+        """
+        :return: None
+        """
+        self.priority = velbus.FIRMWARE_PRIORITY
+
     def needs_no_data(self, data):
         """
-        @return: None
+        :return: None
         """
-        if len(data) != 0:
+        length = len(data)
+        if length != 0:
             self.parser_error("has data included")
 
     def needs_data(self, data, length):
         """
-        @return: None
+        :return: None
         """
         assert isinstance(data, bytes)
-        if len(data) != length:
+        if len(data) < length:
             self.parser_error("needs " + str(length) + " bytes of data")
 
     def needs_fixed_byte(self, byte, value):
         """
-        @return: None
+        :return: None
         """
         assert isinstance(byte, int)
         assert isinstance(value, int)
@@ -232,7 +264,7 @@ class Message(object):
 
     def needs_one_channel(self, channels):
         """
-        @return: None
+        :return: None
         """
         assert isinstance(channels, list)
         if len(channels) != 1 or not isinstance(channels[0], int) or \
